from .base import Entry
from .english_entry import EnglishEntry, Forms
from .japanese_entry import JapaneseEntry, JapaneseReading, Conjugation

__all__ = [
    "Entry",
    "EnglishEntry",
    "Forms",
    "JapaneseEntry",
    "JapaneseReading",
    "Conjugation",
]
